# 记号类型
# EOF (end-of-file) 这个记号用于表示没有更多的输入内容可以进行词法分析

INTEGER, PLUS, EOF = 'INTEGER', 'PLUS', 'EOF'  # 整数，加法，结束标识


class Token:  # 定义记号类
    def __init__(self, value_type, value):  # 定义构造方法
        self.value_type = value_type  # 记号中值的类型（INTEGER, PLUS, EOF ）
        self.value = value  # 记号中的值（0~9，'+'，None ）

    def __str__(self):  # 重写查看记号内容的方法
        '''
        类示例的字符串表示（representation）。
        示例:
            Token(INTEGER, 3)
            Token(PLUS '+')
        '''
        return 'Token({char_type},{value})'.format(char_type=self.value_type, value=self.value)

    def __repr__(self):  # 也可以写成 __repr__=__str__
        return self.__str__()


class Interpreter:  # 定义解释器类
    def __init__(self, text):  # 定义构造方法获取用户输入的表达式
        self.text = text  # 用户输入的表达式
        self.position = 0  # 获取表达式中每一个字符时的位置
        self.current_token = None  # 临时保存记号的变量

    def error(self):  # 定义提示错误的方法
        raise Exception('警告：错误的输入内容！')  # 抛出异常

    def get_next_token(self):  # 定义获取记号的方法
        '''
        词法分析器Lexical analyzer（也称为扫描器scanner或分词器tokenizer）
        此方法负责将字符串划分为记号，1次返回1个记号。
        '''
        text = self.text
        if self.position >= len(text):  # 如果获取字符的位置已经到达末端
            return Token(EOF, None)  # 返回结束标识的记号对象
        current_char = text[self.position]  # 获取当前位置的字符
        if current_char.isdigit():  # 如果当前位置的字符是数字
            token = Token(INTEGER, int(current_char))  # 实例化整数的记号对象
            self.position += 1  # 获取字符的位置自增1，以便获取下一个字符。
            return token  # 返回记号对象
        if current_char == '+':  # 如果当前位置的字符是加号
            token = Token(PLUS, current_char)  # 实例化加号运算符的记号对象
            self.position += 1  # 获取字符的位置自增1，以便获取下一个字符。
            return token  # 返回记号对象
        self.error()  # 如果以上没有任何对象返回，抛出异常。

    def eat(self, token_type):  # 定义辅助运算的方法，此方法用于验证记号对象的值类型是否符合运算要求。
        '''
        验证当前的记号是不是符合运算要求的类型，如果符合要求就“吃掉”当前的记号，获取下一个记号保存到临时保存记号的变量中。
        '''
        if self.current_token.value_type == token_type:  # 如果记号中的值类型符合运算要求
            self.current_token = self.get_next_token()  # 获取下一个记号对象存入变量
        else:  # 否则
            self.error()  # 抛出异常

    def expr(self):  # 定义验证运算结构并计算结果的方法
        '''
        表达式结构为：INTEGER PLUS INTEGER
        '''
        self.current_token = self.get_next_token()  # 获取第一个记号
        left = self.current_token  # 保存第1个记号到变量
        self.eat(INTEGER)  # 调用验证方法传入运算要求的类型

        operator = self.current_token  # 保存第3个记号到变量
        self.eat(PLUS)  # 调用验证方法传入运算要求的类型

        right = self.current_token  # 保存第2个记号到变量
        self.eat(INTEGER)  # 调用验证方法传入运算要求的类型

        result = left.value + right.value  # 进行加法运算获取结果
        return result  # 返回计算结果


def main():
    while True:  # 循环获取输入
        try:
            text = input('>>>')  # 获取用户输入
        except EOFError:  # 捕获到末端错误时退出
            break
        if not text:  # 如果未输入时继续提示输入
            continue
        interpreter = Interpreter(text)  # 实例化解释器对象
        result = interpreter.expr()  # 执行运算方法获取运算结果
        print(text, '=', result)  #


if __name__ == '__main__':
    main()
